﻿namespace AutoSPInstallerGUI
{   
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Reflection;
    using System.Text.RegularExpressions;
    using System.Windows;
    using System.Windows.Documents;
    using System.Windows.Navigation;
    using System.Xml;
    using AutoSPInstallerGUI.Classes;
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Configuration config = new Configuration();

        public MainWindow()
        {
            #if DEBUG
                AutoSPInstallerGUI.Classes.BindingErrorTraceListener.SetTrace();
            #endif

            this.InitializeComponent();
            this.CheckForUpdates();

            System.Windows.Controls.ToolTipService.ShowDurationProperty.OverrideMetadata(typeof(DependencyObject), new FrameworkPropertyMetadata(Int32.MaxValue));

            this.Title = this.Title + " (" + Assembly.GetExecutingAssembly().GetName().Version.ToString() + ")";

            tc_Main.IsEnabled = false;

            if (App.mArgs != null && App.mArgs.Count() > 0)
            {
                if (System.IO.Path.GetExtension(App.mArgs[0]) == ".xml")
                {
                    this.LoadXml(App.mArgs[0]);
                }
                else
                {
                    this.SplashScreenFix();
                    MessageBox.Show("This is not a supported file, it must be an XML file", "Error");
                }
            }
        }

        #region Menu
        private void LoadXml(string fileLocation)
        {
            try
            {
                // Supported XML Version
                string supportedXML = "3.93";

                // Initializes the Config
                this.config = Configuration.LoadFromFile(fileLocation);

                // Check if we have loaded the right file
                if (this.config.Version == null)
                {
                    this.SplashScreenFix();
                    MessageBox.Show("It appears that you have not loaded the correct configuration XML", "Error");
                    return;
                }

                // Check XML Version Number
                if (this.config.Version != supportedXML)
                {
                    this.SplashScreenFix();
                    MessageBox.Show("Please note that this version of AutoSPInstallerGUI has only been tested on AutoSPInstaller XML Version " + supportedXML + "\nYour version is: " + this.config.Version + "\nPlease download the latest/older version of AutoSPInstaller from Source Code", "Error");
                    return;
                }

                // Sets the DataContext for the Main Window
                this.DataContext = this.config;

                // Set up controls
                this.tc_Main.IsEnabled = true;
                this.Menu_SaveXML.IsEnabled = true;
                this.Menu_SaveASXML.IsEnabled = true;
                this.txt_StatusBar.Content = fileLocation;
                if (this.dd_WebApplications.Items.Count > 0)
                {
                    this.dd_WebApplications.SelectedIndex = 0;    
                }

                if (this.dd_Farm_ManagedAccounts.Items.Count > 0)
                {
                    this.dd_Farm_ManagedAccounts.SelectedIndex = 0;
                }
                
            }
            catch (Exception ex)
            {
                this.SplashScreenFix();
                MessageBox.Show("There seems to be an issue with the XML file, please check that you are loading the correct file and that it has no issues:\n\n\n" + ex.ToString(), "Error");
            }
        }

        private void SaveXml(string fileLocation)
        {
            try
            {
                // Save Config to File
                this.config.SaveToFile(fileLocation);

                // Update the StatusBar with the new File Location
                txt_StatusBar.Content = fileLocation;

                // Notify the user that the file was saved
                MessageBox.Show("The XML file was saved to: " + fileLocation);
            }
            catch (Exception ex)
            {
                MessageBox.Show("There was an error saving the XML file: \n" + ex.Message, "Error");
            }
        }

        private void Menu_LoadXML_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.FileName = "AutoSPInstallerInput.xml";
            dlg.DefaultExt = ".xml";
            dlg.Filter = "Extensible Markup Language File (.xml)|*.xml";

            Nullable<bool> result = dlg.ShowDialog();

            if (result == true)
            {
                this.LoadXml(dlg.FileName);
            }
        }

        private void Menu_SaveXML_Click(object sender, RoutedEventArgs e)
        {
            if (this.CheckData())
            {
                this.SaveXml(txt_StatusBar.Content.ToString());
            }
        }

        private void Menu_SaveASXML_Click(object sender, RoutedEventArgs e)
        {
            if (this.CheckData() == true)
            {
                Microsoft.Win32.SaveFileDialog dlg = new Microsoft.Win32.SaveFileDialog();
                dlg.FileName = string.Empty;
                dlg.DefaultExt = ".xml";
                dlg.Filter = "Extensible Markup Language File (.xml)|*.xml";

                Nullable<bool> result = dlg.ShowDialog();

                if (result == true)
                {
                    this.SaveXml(dlg.FileName);
                }
            }
        }

        private void Menu_Exit_Click(object sender, RoutedEventArgs e)
        {
            Environment.Exit(0);
        }

        private void Menu_Help_Click(object sender, RoutedEventArgs e)
        {
            System.Diagnostics.Process.Start("http://autospinstallergui.codeplex.com");
        }

        private void Menu_About_Click(object sender, RoutedEventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.softlanding.ca/About-Us/Pages/default.aspx");
        }

        #endregion

        #region Misc

        /// <summary>
        /// Used for Data Validation
        /// </summary>
        /// <returns>true if succeeded</returns>
        private bool CheckData()
        {
            // Check if any Web Applications Exist
            if (!this.config.WebApplications.WebApplication.Any())
            {
                MessageBox.Show("AutoSPInsatller will fail if there are no Web Applications. Create at least one Web Application", "Error");
                return false;
            }
                
            foreach (ConfigurationWebApplicationsWebApplication webApplication in this.config.WebApplications.WebApplication)
            {
                //// Check the Web Application Pools to see if a Managed Account is selected
                if (this.config.Farm.ManagedAccounts.All(x => x.Username != webApplication.applicationPoolAccount))
                {
                    this.tc_Main.SelectedItem = this.tab_WebApplications;
                    this.dd_WebApplications.SelectedItem = webApplication;
                    this.dd_WebApplications_SelectionChanged(null, null);
                    MessageBox.Show(string.Format("Please reselect the Application Pool for {0}", webApplication.name), "Error");
                    return false;
                }

                //// Check to see if any Site Collection URL Contains :80 or :443
                if (webApplication.SiteCollections.Any(x => x.siteUrl.EndsWith(":80") || x.siteUrl.EndsWith(":443") || x.siteUrl.EndsWith(":80/") || x.siteUrl.EndsWith(":443/") || x.siteUrl.Contains(":80/") || x.siteUrl.Contains(":443/")))
                {
                    this.tc_Main.SelectedItem = this.tab_WebApplications;
                    this.dd_WebApplications.SelectedItem = webApplication;
                    this.dd_WebApplications_SelectionChanged(null, null);
                    MessageBox.Show("The SiteCollection Site URL should not contain :443 or :80", "Error");
                    return false;
                }

                // Trim the Web Application Url for /
                webApplication.url = webApplication.url.TrimEnd("/".ToCharArray());

                // Trim the Site Collection URls for /
                if (webApplication.SiteCollections.Any())
                {
                    foreach (ConfigurationWebApplicationsWebApplicationSiteCollection siteCollection in webApplication.SiteCollections)
                    {
                        siteCollection.siteUrl = siteCollection.siteUrl.TrimEnd("/".ToCharArray());
                    }
                }

                // Warns the user if a WebApplication is not using Claims in 2013 mode.
                if (this.config.Install.SPVersion == "2013")
                {
                    if (webApplication.useClaims == false)
                    {
                        this.tc_Main.SelectedItem = this.tab_WebApplications;
                        this.dd_WebApplications.SelectedItem = webApplication;
                        this.dd_WebApplications_SelectionChanged(null, null);
                        MessageBox.Show(string.Format(webApplication.name + " is using Classic Authentication.\nPlease Note that Claims Authentication is recommended in SharePoint 2013. Classic Authentication is deprecated. If you know what you're doing you may proceed."), "Warning");
                    }
                }
            }

            //// Check to see if only one WebApplication is using type Portal and MySiteHost
            if (this.config.WebApplications.WebApplication.Count(x => x.type == "Portal") > 1 || this.config.WebApplications.WebApplication.Count(x => x.type == "MySiteHost") > 1)
            {
                this.tc_Main.SelectedItem = this.tab_WebApplications;
                MessageBox.Show("Please make sure that only one WebApplication is Type Portal and MySiteHost\nAll other WebApplications should be type \"Other\"", "Error");
                return false;
            }

            //// If the Search Service is to be provisioned
            if (this.config.ServiceApps.EnterpriseSearchService.Provision == true)
            { 
                //// Check the Enterprise Search Service Instance Username
                if (this.config.Farm.ManagedAccounts.All(x => x.Username != this.config.ServiceApps.EnterpriseSearchService.Account))
                {
                    this.tc_Main.SelectedItem = this.tab_ServiceApplications;
                    this.tc_Farm_ServiceApplications.SelectedItem = this.tab_Search;
                    this.dd_ServiceApps_EnterpriseSearchService_Account.SelectedIndex = -1;
                    MessageBox.Show(string.Format("Please reselect the Service Account for the Enterprise Search Service Instance"), "Error");
                    return false;
                }

                //// Check the Enterprise Search Service Application Application Pool Account
                if (this.config.Farm.ManagedAccounts.All(x => x.Username != this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].ApplicationPool.Account))
                {
                    this.tc_Main.SelectedItem = this.tab_ServiceApplications;
                    this.tc_Farm_ServiceApplications.SelectedItem = this.tab_Search;
                    this.tc_ServiceApps_EnterpriseSearch.SelectedItem = this.tab_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications;
                    this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_ApplicationPool_Account.SelectedIndex = -1;
                    MessageBox.Show(string.Format("Please reselect the Application Pool Account"), "Error");
                    return false;
                }

                //// Check the Enterprise Search Service Application Admin Component Application Pool Account
                if (this.config.Farm.ManagedAccounts.All(x => x.Username != this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AdminComponent.ApplicationPool.Account))
                {
                    this.tc_Main.SelectedItem = this.tab_ServiceApplications;
                    this.tc_Farm_ServiceApplications.SelectedItem = this.tab_Search;
                    this.tc_ServiceApps_EnterpriseSearch.SelectedItem = this.tab_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications;
                    this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_AdminComponent_ApplicationPool_Account.SelectedIndex = -1;
                    MessageBox.Show(string.Format("Please reselect the Admin Component Application Pool Account"), "Error");
                    return false;
                }

                //// Checks for specific versions of SharePoint
                if (this.config.Install.SPVersion == "2010")
                {
                    //// Checks that all of the Search Service Components have at least one Server
                    if (!this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].CrawlComponent.Any() || 
                        !this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].QueryComponent.Any() || 
                        !this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].SearchQueryAndSiteSettingsServers.Any() || 
                        !this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AdminComponent.Server.Any())
                    {
                        this.tc_Main.SelectedItem = this.tab_ServiceApplications;
                        this.tc_Farm_ServiceApplications.SelectedItem = this.tab_Search;
                        this.tc_ServiceApps_EnterpriseSearch.SelectedItem = this.tab_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications;
                        MessageBox.Show(string.Format("Please make sure that all of the Search Topology components contain at least one Server"), "Error");
                        return false;
                    }
                }
                else if (this.config.Install.SPVersion == "2013")
                {
                    //// Checks that all of the Search Service Components have at least one Server
                    if (!this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].CrawlComponent.Any() ||
                        !this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].QueryComponent.Any() ||
                        !this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].SearchQueryAndSiteSettingsServers.Any() ||
                        !this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AdminComponent.Server.Any() ||
                        !this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].IndexComponent.Any() ||
                        !this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].ContentProcessingComponent.Any() ||
                        !this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AnalyticsProcessingComponent.Any())
                    {
                        this.tc_Main.SelectedItem = this.tab_ServiceApplications;
                        this.tc_Farm_ServiceApplications.SelectedItem = this.tab_Search;
                        this.tc_ServiceApps_EnterpriseSearch.SelectedItem = this.tab_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications;
                        MessageBox.Show(string.Format("Please make sure that all of the Search Topology components contain at least one Server"), "Error");
                        return false;
                    }

                    //// Checks that Distributed Cache is not set to false
                    if (this.config.Farm.Services.DistributedCache.Start.ToLower() == "false")
                    {
                        this.tc_Main.SelectedItem = this.tab_Farm;
                        this.tc_Farm.SelectedItem = this.tab_Farm_Services;                        
                        MessageBox.Show(string.Format("Please make sure that Distributed Cache is not set to false"), "Error");
                        return false;
                    }
                }
                else
                {
                    this.tc_Main.SelectedItem = this.tab_Install;
                    MessageBox.Show(string.Format("Please reselect the SharePoint Version"), "Error");
                    return false;
                }
            }

            /// Check that FoundationWebApplication is not set to false
            if (this.config.Farm.Services.FoundationWebApplication.Start.ToLower() == "false")
            {
                this.tc_Main.SelectedItem = this.tab_Farm;
                this.tc_Farm.SelectedItem = this.tab_Farm_Services;
                MessageBox.Show(string.Format("Please make sure that WebApplication is not set to false"), "Error");
                return false;
            }

            // Used to check the PassPhrase complexity
            // Code borrowed from AutoSPInstaller
            if (this.config.Farm.Passphrase != string.Empty)
            {
                string password = this.config.Farm.Passphrase;
                int count = 0;
                if (Regex.IsMatch(password, "[a-z]")) { count++; }
                if (Regex.IsMatch(password, "[A-Z]")) { count++; }
                if (Regex.IsMatch(password, "[0-9]")) { count++; }
                if (Regex.IsMatch(password, "[^a-zA-Z0-9]")) { count++; }

                if (count < 3 || password.Length < 8)
                {
                    this.tc_Main.SelectedItem = this.tab_Farm;
                    this.txt_Farm_Passphrase.Focus();
                    MessageBox.Show("Farm passphrase does not meet complexity requirements.\nIt must be at least 8 characters long and contain three of these types:\nUpper case letters\nLower case letters\nDigits\nOther characters", "Error");
                    return false;
                }
            }

            // Check if Install.ConfigFile is empty
            if (String.IsNullOrEmpty(this.config.Install.ConfigFile))
            {
                this.tc_Main.SelectedItem = this.tab_Install;
                MessageBox.Show("The Script will fail if the Config File textbox is empty. Set it to config.xml if you're unsure.", "Error");
                return false;
            }

            return true;
        }

        /// <summary>
        /// Checks for new version of the applications by looking at the Codeplex RSS Feed
        /// </summary>
        private void CheckForUpdates()
        {
            try
            {
                string rssurl = "http://autospinstallergui.codeplex.com/project/feeds/rss?projectrssfeed=codeplex://release/autospinstallergui";
                XmlDocument rssXml = new XmlDocument();
                rssXml.Load(rssurl);

                XmlNodeList rssNodeList = rssXml.SelectNodes("rss/channel/item");

                // Gets the newest rss entries title, link and description
                if (rssNodeList != null)
                {
                    XmlNode rssSubNode = rssNodeList[0].SelectSingleNode("title");
                    string title = rssSubNode != null ? rssSubNode.InnerText : string.Empty;
                    rssSubNode = rssNodeList[0].SelectSingleNode("link");
                    string link = rssSubNode != null ? rssSubNode.InnerText : string.Empty;
                    rssSubNode = rssNodeList[0].SelectSingleNode("description");
                    string desc = rssSubNode != null ? rssSubNode.InnerText : string.Empty;

                    // Strips the HTML code from the RSS Description
                    string strippeddesc = Regex.Replace(desc.Replace("<br />", "\n").Replace("&quot;", "\"").Replace("&#39;","'"), "<.*?>", string.Empty);

                    // Gets the server and client version strings
                    string server = title.Substring(title.IndexOf(":", StringComparison.Ordinal) + 2, title.IndexOf("(", StringComparison.Ordinal) - title.IndexOf(":", StringComparison.Ordinal) - 3);
                    string client = Assembly.GetExecutingAssembly().GetName().Version.ToString();

                    // Converts the version string to a List of integers
                    List<int> lstServer = server.Split('.').ToList().ConvertAll(Convert.ToInt32);
                    List<int> lstClient = client.Split('.').ToList().ConvertAll(Convert.ToInt32);

                    // Check if the RSS version contains a version higher than the client
                    for (int i = 0; i <= 3; i++)
                    {
                        if (lstServer[i] < lstClient[i])
                        {
                            return;
                        }
                        else if (lstServer[i] > lstClient[i])
                        {
                            this.SplashScreenFix();
                            if (MessageBox.Show(title + "\n" + strippeddesc + "\n\nClick OK to go to the Download Page.", "Update Found!", MessageBoxButton.OKCancel) == MessageBoxResult.OK)
                            {
                                System.Diagnostics.Process.Start(link);
                                return;
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                if (ex.Message == "The remote name could not be resolved: 'autospinstallergui.codeplex.com'")
                {
                    this.SplashScreenFix();
                    MessageBox.Show("Unable to check for a new version, you most likely don't have internet access.", "Warning");
                }
            }
        }

        /// <summary>
        /// Trims the Textbox text if needed
        /// </summary>
        /// <param name="sender">Textbox</param>
        /// <param name="e"></param>
        private void TrimTextBox(object sender, RoutedEventArgs e)
        {
            System.Windows.Controls.TextBox senderTextBox = (System.Windows.Controls.TextBox)sender;

            if (senderTextBox.Text.EndsWith(" ") || senderTextBox.Text.StartsWith(" "))
            {
                senderTextBox.Text = senderTextBox.Text.Trim();
            }
        }

        /// <summary>
        /// Assists in opening link for TextBlocks
        /// Code from: http://stackoverflow.com/questions/2092890/add-hyperlink-to-textblock-wpf/2093016#2093016
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Hyperlink_RequestNavigate(object sender, RequestNavigateEventArgs e)
        {
            System.Diagnostics.Process.Start(e.Uri.ToString());
        }

        /// <summary>
        /// Fix for SplashScreen closing MsgBoxes
        /// http://connect.microsoft.com/VisualStudio/feedback/details/600197/wpf-splash-screen-dismisses-dialog-box
        /// </summary>
        private void SplashScreenFix()
        {
            Window temp = new Window() { Visibility = Visibility.Hidden };
            temp.Show();
        }
        #endregion

        #region Farm Tab
        private void rdo_Install_SPVersion_2010_Checked(object sender, RoutedEventArgs e)
        {
            this.dd_WebApplications_SiteCollections_Template.ItemsSource = this.config.WebTemplates.Where(x => x.CompatibilityLevel == 14);
        }

        private void rdo_Install_SPVersion_2013_Checked(object sender, RoutedEventArgs e)
        {
            this.dd_WebApplications_SiteCollections_Template.ItemsSource = this.config.WebTemplates.GroupBy(x => x.Name).Select(grp => grp.First());
        }

        private void txt_Farm_ManagedAccounts_Password_TextChanged(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {
            this.config.SetEnterpriseSearchServicePassword();
            this.config.SetEnterpriseSearchServiceApplicationPassword();
        }
        
        private void dd_Farm_ManagedAccounts_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            if (this.dd_Farm_ManagedAccounts.SelectedIndex != -1 && this.config.Farm.ManagedAccounts[this.dd_Farm_ManagedAccounts.SelectedIndex].CommonName == "spservice")
            {
                this.lbl_Farm_ManagedAccounts_Note.Text = "Please Note: This is the Service Account which will be used as the Application Pool Account for most of the Service Applications. By default, only Search uses a separate Managed Account.";
            }
            else
            {
                this.lbl_Farm_ManagedAccounts_Note.Text = string.Empty;
            }
        }
        
        private void btn_Farm_ManagedAccounts_Add_Click(object sender, RoutedEventArgs e)
        {
            ConfigurationFarmManagedAccount managedAccount = new ConfigurationFarmManagedAccount();
            managedAccount.Username = "Domain\\TempAccount" + this.config.Farm.ManagedAccounts.Count().ToString();
            managedAccount.Password = "Change Me";
            managedAccount.CommonName = "Other";
            this.config.Farm.ManagedAccounts.Add(managedAccount);
            this.dd_Farm_ManagedAccounts.SelectedIndex = this.dd_Farm_ManagedAccounts.Items.Count - 1;          
        }

        private void btn_Farm_ManagedAccounts_Remove_Click(object sender, RoutedEventArgs e)
        {
            if (this.dd_Farm_ManagedAccounts.SelectedIndex != -1)
            {
                if (this.config.Farm.ManagedAccounts[this.dd_Farm_ManagedAccounts.SelectedIndex].CommonName != null && this.config.Farm.ManagedAccounts[this.dd_Farm_ManagedAccounts.SelectedIndex].CommonName.ToLower() == "spservice")
                {
                    MessageBox.Show("This Account is required and should not be deleted. However, you can change the Account Info", "Warning");
                }
                else
                {
                    this.config.Farm.ManagedAccounts.RemoveAt(this.dd_Farm_ManagedAccounts.SelectedIndex);
                    this.dd_Farm_ManagedAccounts.SelectedIndex = this.dd_Farm_ManagedAccounts.Items.Count - 1;
                }
            }
        }
        #endregion

        #region Web Applications
        private void dd_WebApplications_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            if (dd_WebApplications_ManagedPaths.Items.Count > 0)
            {
                dd_WebApplications_ManagedPaths.SelectedIndex = 0;
            }

            if (dd_WebApplications_SiteCollections.Items.Count > 0)
            {
                dd_WebApplications_SiteCollections.SelectedIndex = 0;
            }
        }

        private void btn_WebApplications_Add_Click(object sender, RoutedEventArgs e)
        {
            ConfigurationWebApplicationsWebApplication webApplication = new ConfigurationWebApplicationsWebApplication();
            webApplication.type = "Other";
            webApplication.name = "New" + this.config.WebApplications.WebApplication.Count().ToString();
            webApplication.applicationPool = "WebApplication Pool";
            webApplication.applicationPoolAccount = string.Empty;
            webApplication.url = "http://localhost";
            webApplication.port = (80 + this.config.WebApplications.WebApplication.Count()).ToString();
            webApplication.databaseName = "WSS_Content" + this.config.WebApplications.WebApplication.Count().ToString();
            webApplication.UseHostHeader = true;
            webApplication.AddURLToLocalIntranetZone = true;
            webApplication.useClaims = true;
            webApplication.useBasicAuthentication = false;
            webApplication.useOnlineWebPartCatalog = false;
            webApplication.Database = new ConfigurationWebApplicationsWebApplicationDatabase();
            webApplication.Database.DBServer = string.Empty;
            webApplication.Database.DBAlias = new ConfigurationWebApplicationsWebApplicationDatabaseDBAlias();
            webApplication.Database.DBAlias.Create = false;
            webApplication.Database.DBAlias.DBInstance = "SERVER\\INSTANCE";
            webApplication.Database.DBAlias.DBPort = string.Empty;

            webApplication.ManagedPaths = new System.Collections.ObjectModel.ObservableCollection<ConfigurationWebApplicationsWebApplicationManagedPath>();
            webApplication.SiteCollections = new System.Collections.ObjectModel.ObservableCollection<ConfigurationWebApplicationsWebApplicationSiteCollection>();

            if (this.config.WebApplications.WebApplication == null)
            {
                this.config.WebApplications.WebApplication = new System.Collections.ObjectModel.ObservableCollection<ConfigurationWebApplicationsWebApplication>();
            }

            this.config.WebApplications.WebApplication.Add(webApplication);
            this.dd_WebApplications.SelectedIndex = dd_WebApplications.Items.Count - 1;
        }

        private void btn_WebApplications_Remove_Click(object sender, RoutedEventArgs e)
        {
            if (dd_WebApplications.SelectedIndex != -1)
            {
                this.config.WebApplications.WebApplication.RemoveAt(this.dd_WebApplications.SelectedIndex);
                this.dd_WebApplications.SelectedIndex = this.dd_WebApplications.Items.Count - 1;
            }
        }

        private void btn_WebApplications_ManagedPaths_Add_Click(object sender, RoutedEventArgs e)
        {
                ConfigurationWebApplicationsWebApplicationManagedPath managedPath = new ConfigurationWebApplicationsWebApplicationManagedPath();
                managedPath.relativeUrl = "ChangeMe" + dd_WebApplications_ManagedPaths.Items.Count.ToString();
                managedPath.@explicit = false;

                this.config.WebApplications.WebApplication[this.dd_WebApplications.SelectedIndex].ManagedPaths.Add(managedPath);
                this.dd_WebApplications_ManagedPaths.SelectedIndex = this.dd_WebApplications_ManagedPaths.Items.Count - 1;
        }

        private void btn_WebApplications_ManagedPaths_Remove_Click(object sender, RoutedEventArgs e)
        {
            if (dd_WebApplications_ManagedPaths.SelectedIndex != -1)
            {
                this.config.WebApplications.WebApplication[this.dd_WebApplications.SelectedIndex].ManagedPaths.RemoveAt(this.dd_WebApplications_ManagedPaths.SelectedIndex);
                dd_WebApplications_ManagedPaths.SelectedIndex = dd_WebApplications_ManagedPaths.Items.Count - 1;
            }
        }
        
        private void btn_WebApplications_SiteCollections_Add_Click(object sender, RoutedEventArgs e)
        {
            ConfigurationWebApplicationsWebApplicationSiteCollection siteCollection = new ConfigurationWebApplicationsWebApplicationSiteCollection();
            siteCollection.siteUrl = this.config.WebApplications.WebApplication[this.dd_WebApplications.SelectedIndex].url.TrimEnd('/') + "/sites/Site" + this.config.WebApplications.WebApplication[this.dd_WebApplications.SelectedIndex].SiteCollections.Count.ToString();
            siteCollection.Owner = "Domain\\ChangeMe";
            siteCollection.HostNamedSiteCollection = false;
            siteCollection.Name = "New Site";
            siteCollection.Description = "New Site";
            siteCollection.SearchUrl = "http://localhost/search";
            siteCollection.CustomTemplate = false;
            siteCollection.Template = "STS#0";
            siteCollection.LCID = "1033";
            siteCollection.Locale = "en-us";
            siteCollection.Time24 = false;

            this.config.WebApplications.WebApplication[this.dd_WebApplications.SelectedIndex].SiteCollections.Add(siteCollection);
            this.dd_WebApplications_SiteCollections.SelectedIndex = this.dd_WebApplications_SiteCollections.Items.Count - 1;
        }

        private void btn_WebApplications_SiteCollections_Remove_Click(object sender, RoutedEventArgs e)
        {
            if (dd_WebApplications_SiteCollections.SelectedIndex != -1)
            {
                this.config.WebApplications.WebApplication[this.dd_WebApplications.SelectedIndex].SiteCollections.RemoveAt(this.dd_WebApplications_SiteCollections.SelectedIndex);
                this.dd_WebApplications_SiteCollections.SelectedIndex = this.dd_WebApplications_SiteCollections.Items.Count - 1;
            }
        }

        #endregion

        #region Service Applications
        private void dd_ServiceApps_EnterpriseSearchService_Account_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            this.config.SetEnterpriseSearchServicePassword();
        }

        private void dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_ApplicationPool_Account_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            this.config.SetEnterpriseSearchServiceApplicationPassword();
        }
        
        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_CrawlComponent_Add_Click(object sender, RoutedEventArgs e)
        {
            ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationServer crawlComponent = new ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationServer();
            crawlComponent.Name = "Server Name" + this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].CrawlComponent.Count.ToString();

            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].CrawlComponent.Add(crawlComponent);
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_CrawlComponent.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].CrawlComponent.Count - 1;
        }
        
        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_CrawlComponent_Remove_Click(object sender, RoutedEventArgs e)
        {
            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].CrawlComponent.RemoveAt(this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_CrawlComponent.SelectedIndex);
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_CrawlComponent.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].CrawlComponent.Count - 1;
        }

        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_QueryComponent_Add_Click(object sender, RoutedEventArgs e)
        {
            ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationServer1 queryComponent = new ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationServer1();
            queryComponent.Name = "Server Name" + this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].QueryComponent.Count.ToString();
            
            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].QueryComponent.Add(queryComponent);
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_QueryComponent.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].QueryComponent.Count - 1;
        }

        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_QueryComponent_Remove_Click(object sender, RoutedEventArgs e)
        {
            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].QueryComponent.RemoveAt(this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_QueryComponent.SelectedIndex);
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_QueryComponent.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].QueryComponent.Count - 1;
        }

        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_SearchQueryAndSiteSettingsServers_Add_Click(object sender, RoutedEventArgs e)
        {
            ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationServer2 searchQueryAndSiteSettingsServers = new ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationServer2();
            searchQueryAndSiteSettingsServers.Name = "Server Name" + this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].SearchQueryAndSiteSettingsServers.Count.ToString();

            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].SearchQueryAndSiteSettingsServers.Add(searchQueryAndSiteSettingsServers);           
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_SearchQueryAndSiteSettingsServers.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].SearchQueryAndSiteSettingsServers.Count - 1;
        }

        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_SearchQueryAndSiteSettingsServers_Remove_Click(object sender, RoutedEventArgs e)
        {
            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].SearchQueryAndSiteSettingsServers.RemoveAt(this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_SearchQueryAndSiteSettingsServers.SelectedIndex);
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_SearchQueryAndSiteSettingsServers.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].SearchQueryAndSiteSettingsServers.Count - 1;
        }

        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_AdminComponent_Add_Click(object sender, RoutedEventArgs e)
        {
            ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationAdminComponentServer adminComponent = new ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationAdminComponentServer();
            adminComponent.Name = "Server Name" + this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AdminComponent.Server.Count.ToString();

            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AdminComponent.Server.Add(adminComponent);
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_AdminComponent.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AdminComponent.Server.Count - 1;
        }

        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_AdminComponent_Remove_Click(object sender, RoutedEventArgs e)
        {
            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AdminComponent.Server.RemoveAt(this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_AdminComponent.SelectedIndex);
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_AdminComponent.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AdminComponent.Server.Count - 1;
        }

        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_IndexComponent_Add_Click(object sender, RoutedEventArgs e)
        {
            ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationServer3 indexComponent = new ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationServer3();
            indexComponent.Name = "Server Name" + this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].IndexComponent.Count.ToString();

            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].IndexComponent.Add(indexComponent);
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_IndexComponent.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].IndexComponent.Count - 1;
        }

        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_IndexComponent_Remove_Click(object sender, RoutedEventArgs e)
        {
            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].IndexComponent.RemoveAt(this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_IndexComponent.SelectedIndex);
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_IndexComponent.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].IndexComponent.Count - 1;
        }

        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_ContentProcessingComponent_Add_Click(object sender, RoutedEventArgs e)
        {
            ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationServer4 contentProcessingComponent = new ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationServer4();
            contentProcessingComponent.Name = "Server Name" + this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].ContentProcessingComponent.Count.ToString();

            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].ContentProcessingComponent.Add(contentProcessingComponent);
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_ContentProcessingComponent.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].ContentProcessingComponent.Count - 1;          
        }

        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_ContentProcessingComponent_Remove_Click(object sender, RoutedEventArgs e)
        {
            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].ContentProcessingComponent.RemoveAt(this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_ContentProcessingComponent.SelectedIndex);
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_ContentProcessingComponent.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].ContentProcessingComponent.Count - 1;          
        }

        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_AnalyticsProcessingComponent_Add_Click(object sender, RoutedEventArgs e)
        {
            ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationServer5 analyticsProcessingComponent = new ConfigurationServiceAppsEnterpriseSearchServiceEnterpriseSearchServiceApplicationServer5();
            analyticsProcessingComponent.Name = "Server Name" + this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AnalyticsProcessingComponent.Count.ToString();

            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AnalyticsProcessingComponent.Add(analyticsProcessingComponent);
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_AnalyticsProcessingComponent.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AnalyticsProcessingComponent.Count - 1;             
        }

        private void btn_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_AnalyticsProcessingComponent_Remove_Click(object sender, RoutedEventArgs e)
        {
            this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AnalyticsProcessingComponent.RemoveAt(this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_AnalyticsProcessingComponent.SelectedIndex);
            this.dd_ServiceApps_EnterpriseSearchService_EnterpriseSearchServiceApplications_AnalyticsProcessingComponent.SelectedIndex = this.config.ServiceApps.EnterpriseSearchService.EnterpriseSearchServiceApplications[0].AnalyticsProcessingComponent.Count - 1;             
        }
        #endregion
    }
}